--[[

$Revision: 33 $

(C) Copyright 2008 Bethink (bethink at naef dot com)
See LICENSE.txt for license terms.

]]


----------------------------------------------------------------------
-- Acquire libraries and declare add-on

local L = LibStub("AceLocale-3.0"):GetLocale("QuoteNoMore", true)
QuoteNoMore = LibStub("AceAddon-3.0"):NewAddon("QuoteNoMore",
	"AceConsole-3.0",
	"AceHook-3.0")


----------------------------------------------------------------------
-- Constants

-- Channel map
local CHANNEL_MAP = {
	SAY = "filterSay",
	YELL = "filterYell",
	WHISPER = "filterWhisper",
	GUILD = "filterGuild",
	OFFICER = "filterOfficer",
	PARTY = "filterParty",
	RAID = "filterRaid",
	RAID_LEADER = "filterRaidLeader",
	BATTLEGROUND = "filterBattleground",
	BATTLEGROUND_LEADER = "filterBattlegroundLeader"
}
	
-- Ace options
local ACE_OPTS = {
	type = "group",
	name = "QuoteNoMore",
	handler = QuoteNoMore,
	args = {
		filter = {
			type = "group",
			name = L["FILTER"],
			desc = L["FILTER_DESC"],
			args = {
				say = {
					type = "toggle",
					name = L["SAY"],
					desc = L["SAY_DESC"],
					get = "GetFilterSay",
					set = "SetFilterSay"
				},
				yell = {
					type = "toggle",
					name = L["YELL"],
					desc = L["YELL_DESC"],
					get = "GetFilterYell",
					set = "SetFilterYell"
				},
				whisper = {
					type = "toggle",
					name = L["WHISPER"],
					desc = L["WHISPER_DESC"],
					get = "GetFilterWhisper",
					set = "SetFilterWhisper"
				},
				guild = {
					type = "toggle",
					name = L["GUILD"],
					desc = L["GUILD_DESC"],
					get = "GetFilterGuild",
					set = "SetFilterGuild"
				},
				officer = {
					type = "toggle",
					name = L["OFFICER"],
					desc = L["OFFICER_DESC"],
					get = "GetFilterOfficer",
					set = "SetFilterOfficer"
				},
				party = {
					type = "toggle",
					name = L["PARTY"],
					desc = L["PARTY_DESC"],
					get = "GetFilterParty",
					set = "SetFilterParty"
				},
				raid = {
					type = "toggle",
					name = L["RAID"],
					desc = L["RAID_DESC"],
					get = "GetFilterRaid",
					set = "SetFilterRaid"
				},
				raidLeader = {
					type = "toggle",
					name = L["RAID_LEADER"],
					desc = L["RAID_LEADER_DESC"],
					get = "GetFilterRaidLeader",
					set = "SetFilterRaidLeader"
				},
				battleground = {
					type = "toggle",
					name = L["BATTLEGROUND"],
					desc = L["BATTLEGROUND_DESC"],
					get = "GetFilterBattleground",
					set = "SetFilterBattleground"
				},
				battlegroundLeader = {
					type = "toggle",
					name = L["BATTLEGROUND_LEADER"],
					desc = L["BATTLEGROUND_LEADER_DESC"],
					get = "GetFilterBattlegroundLeader",
					set = "SetFilterBattlegroundLeader"
				}
			}
		}
	}
}

-- History
local HISTORY_MAX = 10


----------------------------------------------------------------------
-- Core handlers

-- One-time initialization
function QuoteNoMore:OnInitialize ()
	-- Options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("QuoteNoMore", ACE_OPTS, L["SLASH_COMMANDS"])
	self.options = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QuoteNoMore", "QuoteNoMore")
	
	-- Database
	self.db = LibStub("AceDB-3.0"):New("QuoteNoMoreDB")
	self.db:RegisterDefaults({
		profile = {
			filterSay = true,
			filterYell = true,
			filterWhisper = true,
			filterGuild = true,
			filterOfficer = true,
			filterParty = true,
			filterRaid = true,
			filterRaidLeader = true,
			filterBattleground = true,
			filterBattlegroundLeader = true,
			showCount = true
		}
	})

	-- Status
	self.filterCount = 0
	self.history = { }
	
	-- LDB
	self.dataObject = LibStub("LibDataBroker-1.1"):NewDataObject("QuoteNoMore", { 
		type = "data source",
		text = "",
		icon = "Interface\\Icons\\Ability_Stealth.blp",
		OnClick = function (frame, button)
			if button == "RightButton" then
				QuoteNoMore:ShowOptions()
			end
		end,
		OnTooltipShow = function (tt)
			QuoteNoMore:OnTooltipShow(tt)
		end })
	self:Update()
end

-- Handles enabling of this add-on 
function QuoteNoMore:OnEnable ()
	self:RawHook("ChatFrame_MessageEventHandler", true)
end


----------------------------------------------------------------------
-- Settings

-- Returns whether to filter CHAT_MSG_SAY
function QuoteNoMore:GetFilterSay ()
	return self.db.profile.filterSay
end

-- Sets whether to filter CHAT_MSG_SAY
function QuoteNoMore:SetFilterSay (filterSay)
	self.db.profile.filterSay = filterSay
end

-- Returns whether to filter CHAT_MSG_YELL
function QuoteNoMore:GetFilterYell ()
	return self.db.profile.filterYell
end

-- Sets whether to filter CHAT_MSG_YELL
function QuoteNoMore:SetFilterYell (filterYell)
	self.db.profile.filterYell = filterYell
end

-- Returns whether to filter CHAT_MSG_WHISPER
function QuoteNoMore:GetFilterWhisper ()
	return self.db.profile.filterWhisper
end

-- Sets whether to filter CHAT_MSG_WHISPER
function QuoteNoMore:SetFilterWhisper (filterWhisper)
	self.db.profile.filterWhisper = filterWhisper
end

-- Returns whether to filter CHAT_MSG_GUILD
function QuoteNoMore:GetFilterGuild ()
	return self.db.profile.filterGuild
end

-- Sets whether to filter CHAT_MSG_GUILD
function QuoteNoMore:SetFilterGuild (filterGuild)
	self.db.profile.filterGuild = filterGuild
end

-- Returns whether to filter CHAT_MSG_OFFICER
function QuoteNoMore:GetFilterOfficer ()
	return self.db.profile.filterOfficer
end

-- Sets whether to filter CHAT_MSG_OFFICER
function QuoteNoMore:SetFilterOfficer (filterOfficer)
	self.db.profile.filterOfficer = filterOfficer
end

-- Returns whether to filter CHAT_MSG_PARTY
function QuoteNoMore:GetFilterParty ()
	return self.db.profile.filterParty
end

-- Sets whether to filter CHAT_MSG_PARTY
function QuoteNoMore:SetFilterParty (filterParty)
	self.db.profile.filterParty = filterParty
end

-- Returns whether to filter CHAT_MSG_RAID
function QuoteNoMore:GetFilterRaid ()
	return self.db.profile.filterRaid
end

-- Sets whether to filter CHAT_MSG_RAID
function QuoteNoMore:SetFilterRaid (filterRaid)
	self.db.profile.filterRaid = filterRaid
end

-- Returns whether to filter CHAT_MSG_RAID_LEADER
function QuoteNoMore:GetFilterRaidLeader()
	return self.db.profile.filterRaidLeader
end

-- Sets whether to filter CHAT_MSG_RAID_LEADER
function QuoteNoMore:SetFilterRaidLeader (filterRaidLeader)
	self.db.profile.filterRaidLeader = filterRaidLeader
end

-- Returns whether to filter CHAT_MSG_BATTLEGROUND
function QuoteNoMore:GetFilterBattleground ()
	return self.db.profile.filterBattleground
end

-- Sets whether to filter CHAT_MSG_BATTLEGROUND
function QuoteNoMore:SetFilterBattleground (filterBattleground)
	self.db.profile.filterBattleground = filterBattleground
end

-- Returns whether to filter CHAT_MSG_BATTLEGROUND_LEADER
function QuoteNoMore:GetFilterBattlegroundLeader ()
	return self.db.profile.filterBattlegroundLeader
end

-- Sets whether to filter CHAT_MSG_BATTLEGROUND_LEADER
function QuoteNoMore:SetFilterBattlegroundLeader (filterBattlegroundLeader)
	self.db.profile.filterBattlegroundLeader = filterBattlegroundLeader
end

-- Returns whether to show the number of filtered messages
function QuoteNoMore:GetShowCount ()
	return self.db.profile.showCount
end

-- Sets whether to show the number of filtered messages
function QuoteNoMore:SetShowCount (showCount)
	self.db.profile.showCount = showCount
	self:UpdateText()
end


----------------------------------------------------------------------
-- LDB

-- Shows the options
function QuoteNoMore:ShowOptions ()
	InterfaceOptionsFrame_OpenToCategory(self.options)
end

-- Updates the data object
function QuoteNoMore:Update ()
	self.dataObject.text = tostring(self.filterCount)
end

--  Handles tooltip updates
function QuoteNoMore:OnTooltipShow (tt)
	tt:AddLine("QuoteNoMore")
	tt:AddLine(" ")
	tt:AddLine(L["MOST_RECENT_FILTERED_MESSAGES"], 1.0, 1.0, 1.0)
	for _, chatMessage in ipairs(self.history) do
		tt:AddDoubleLine(chatMessage.author, chatMessage.message, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5)
	end
end


----------------------------------------------------------------------
-- Filter

-- Filters chat message events
function QuoteNoMore:ChatFrame_MessageEventHandler (frame, event, arg1, arg2, ...)
	-- Check whether to apply the filter
	if strsub(event, 1, 8) == "CHAT_MSG" then
		local type = strsub(event, 10)
		if CHANNEL_MAP[type] then
			local filter = self.db.profile[CHANNEL_MAP[type]]
			if filter then
				if arg1 and string.find(arg1, "^%[ .+ %] %(~%d+~%)$") then
					self:ProcessFilteredChatMessage(arg2, arg1)
					return
				end
			end
		end
	end
	
	-- Call original method
	return self.hooks["ChatFrame_MessageEventHandler"](frame, event, arg1, arg2, ...)
end

-- Processes a filtered chat message
function QuoteNoMore:ProcessFilteredChatMessage (author, message)
	-- Count
	self.filterCount = self.filterCount + 1
	
	-- History
	local chatMessage = {
		author = author,
		message = message
	}
	if #self.history == HISTORY_MAX then
		table.remove(self.history, 1)
	end
	table.insert(self.history, chatMessage)
	
	-- Update
	self:Update()
end